/**
 * @file rem_audio.h  Wrapper for all Audio header files
 *
 * Copyright (C) 2010 Creytiv.com
 */


#include "rem_au.h"
#include "rem_aulevel.h"
#include "rem_auframe.h"
#include "rem_aubuf.h"
#include "rem_auconv.h"
#include "rem_aufile.h"
#include "rem_autone.h"
#include "rem_aumix.h"
#include "rem_dtmf.h"
#include "rem_fir.h"
#include "rem_goertzel.h"
#include "rem_auresamp.h"
#include "rem_g711.h"
#include "rem_aac.h"
