/**
 * @file rem.h  Wrapper for librem headers
 *
 * Copyright (C) 2010 Creytiv.com
 */

#ifndef REM_H__
#define REM_H__

#ifdef __cplusplus
extern "C" {
#endif


#include "rem_audio.h"
#include "rem_video.h"
#include "rem_dsp.h"
#include "rem_flv.h"


#ifdef __cplusplus
}
#endif

#endif
